package controllers;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import services.FriendDAO;
import services.NotificationDAO;
import services.UserDAO;
import utils.Security;
import utils.ValidationUtils;
import dto.FriendAndBlocked;
import dto.User;
import exceptions.UnauthorizedException;
import exceptions.ValidationException;


@Controller
@RequestMapping(value="/friends")
public class FriendsController {
	
	private @Autowired FriendDAO friendDAO;
	private @Autowired UserDAO userDAO;
	private @Autowired NotificationDAO notificationDAO;
	
	@Transactional(readOnly = false)
	@RequestMapping(value="/create", method=RequestMethod.POST)
	@ResponseBody
	public void newFriend(String userSid1, String userSid2, String hash) throws UnauthorizedException, IOException, ValidationException{
		if (!ValidationUtils.isEmptyOrNull(userSid1)
				&& !ValidationUtils.isEmptyOrNull(userSid2)
				&& !ValidationUtils.isEmptyOrNull(hash)){	
			try{
				if (Security.existHash(hash, userDAO)){
					if (!friendDAO.exists(userSid1, userSid2)){
						 friendDAO.create(userSid1, userSid2);
						 // Creates a notification
						 notificationDAO.createNewFriend(userSid2, userSid1);
					}
					else{
						throw new UnauthorizedException("Not friends allowed");
					}
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/me", method=RequestMethod.GET)
	@ResponseBody
	public List<FriendAndBlocked> getFriends(String hash) throws IOException, UnauthorizedException, ValidationException{
		if (!ValidationUtils.isEmptyOrNull(hash)){	
			try{
				if (Security.existHash(hash, userDAO)){
					List<FriendAndBlocked> friendAndBlockedList = new ArrayList<FriendAndBlocked>();
					
					String userSid = Security.getUsername(hash);
					List<User> userList = userDAO.getFriends(userSid);
					for (User user: userList){
						FriendAndBlocked friendAndBlocked = new FriendAndBlocked();
						friendAndBlocked.setUsername(user.getUsername());
						friendAndBlocked.setBlocked(!ValidationUtils.isEmptyOrNull(friendDAO.getBlocker(userSid, user.getUsername())));
						friendAndBlockedList.add(friendAndBlocked);
					}
					return friendAndBlockedList;
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	@Transactional(readOnly = false)
	@RequestMapping(value="/block", method=RequestMethod.POST)
	@ResponseBody
	public void blockFriend(String userLogged, String userSid2, boolean block, String hash) throws UnauthorizedException, IOException, ValidationException{
		if (!ValidationUtils.isEmptyOrNull(userLogged)
				&& !ValidationUtils.isEmptyOrNull(userSid2)
				&& !ValidationUtils.isEmptyOrNull(hash)){	
			try{
				if (Security.existHash(hash, userDAO)){
					if (friendDAO.exists(userLogged, userSid2)){
						friendDAO.block(userLogged, userSid2, block);
					}
					else{
						throw new UnauthorizedException("Not friends allowed");
					}
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/searchnotfriends", method=RequestMethod.GET)
	@ResponseBody
	public List<User> searchNotFriends(String words, String hash) throws IOException, UnauthorizedException, ValidationException{
		if (!ValidationUtils.isEmptyOrNull(hash)){	
			try{
				if (Security.existHash(hash, userDAO)){
					List<User> userList;
					String userSid = Security.getUsername(hash);
					if (!ValidationUtils.isEmptyOrNull(words)){
						userList = userDAO.getUserNotFriendsOrderAlphabetic(userSid, words);
					}
					else{
						userList = userDAO.getUserNotFriendsOrderAlphabetic(userSid);
					}
					// Puts null params that we don't need
					for (User user : userList){
						user.setEmail(null);
						user.setPassword(null);
						user.setRecordDate(null);
						user.setEndedDate(null);
					}
					return userList;
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	// Errors
	
	@ExceptionHandler(IOException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleIoException(){}
	
	@ExceptionHandler(DataAccessException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleDataAccessException(){}
	
	@ExceptionHandler(UnauthorizedException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleUnauthorizedException(){}
	
	@ExceptionHandler(ValidationException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleValidationException(){}
}