package controllers;

import java.io.IOException;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import services.FilmDAO;
import services.UserDAO;
import utils.Security;
import utils.ValidationUtils;
import dto.Film;
import exceptions.UnauthorizedException;
import exceptions.ValidationException;


@Controller
@RequestMapping(value="/profile")
public class ProfileController {

	private @Autowired FilmDAO filmDAO;
	private @Autowired UserDAO userDAO;
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/films", method=RequestMethod.GET)
	@ResponseBody
	public List<Film> getMyFilms(String hash) throws UnauthorizedException, ValidationException, IOException{
		if (!ValidationUtils.isEmptyOrNull(hash)){	
			try{
				if (Security.existHash(hash, userDAO)){
					String ownerSid = Security.getUsername(hash);
					List<Film> filmList = filmDAO.getUserFilmsOrderRecordDate(ownerSid);
					// Puts null params that we don't need
					for (Film film : filmList){
						film.setOwnerSid(null);
						film.setRecordDate(null);
						film.setInCinema(null);
						film.setPremiere(null);
						film.setSynopsis(null);
						film.setGenre(null);
					}
					return filmList;
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/films/filters", method=RequestMethod.GET)
	@ResponseBody
	public List<Film> getMyFilmsWithFilters(boolean gotIt, Boolean seeWannaSee, String hash) throws UnauthorizedException, ValidationException, IOException{
		if (!ValidationUtils.isEmptyOrNull(hash)){	
			try{
				if (Security.existHash(hash, userDAO)){
					String ownerSid = Security.getUsername(hash);
					List<Film> filmList = filmDAO.getUserFilmsWithFiltersOrderRecordDate(ownerSid, gotIt, seeWannaSee);
					// Puts null params that We don't need
					for (Film film : filmList){
						film.setOwnerSid(null);
						film.setRecordDate(null);
						film.setInCinema(null);
						film.setPremiere(null);
						film.setSynopsis(null);
						film.setGenre(null);
					}
					return filmList;
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	// Errors
	
	@ExceptionHandler(IOException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleIoException(){}
		
	@ExceptionHandler(DataAccessException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleDataAccessException(){}
	
	@ExceptionHandler(UnauthorizedException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleUnauthorizedException(){}
	
	@ExceptionHandler(ValidationException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleValidationException(){}
}
