package services;

import java.util.List;

import org.springframework.dao.DataAccessException;

import dto.User;

public interface UserDAO {
	
	/**
	 * Creates a user
	 * @param username String
	 * @param email String
	 * @param password String
	 * @throws DataAccessException
	 */
	void create(String username, String email, String password) throws DataAccessException;
		
	/**
	 * Gets an user
	 * @param userSid String
 	 * @return User returned
	 * @throws DataAccessException
	 */
	User get(String userSid) throws DataAccessException;
	
	/**
	 * Checks if the user exists
	 * @param username String
	 * @param password String
	 * @return boolean indicates if the user exists
	 * @throws DataAccessException
	 */
	boolean exists(String username, String password) throws DataAccessException;
	
	/**
	 * Checks if the username and email are availables
	 * @param username String
	 * @param email String
	 * @return boolean indicates if the username and email are availables or not
	 * @throws DataAccessException
	 */
	boolean usernameAndEmailAvailable(String username, String email) throws DataAccessException;
	
	/**
	 * Checks if the email is available
	 * @param email String
	 * @return boolean indicates if the email is available or not
	 * @throws DataAccessException
	 */
	boolean emailAvailable(String email) throws DataAccessException;
	
	/**
	 * Gets users ordered alphabetically and not friend yet
	 * @param userSid String
	 * @return List<User> users returned
	 * @throws DataAccessException
	 */
	List<User> getUserNotFriendsOrderAlphabetic(String userSid) throws DataAccessException;
	
	/**
	 * Gets users ordered alphabetically between words and not friend user yet
	 * @param userSid String
	 * @param words String
	 * @return List<User> users returned
	 * @throws DataAccessException
	 */
	List<User> getUserNotFriendsOrderAlphabetic(String userSid, String words) throws DataAccessException;

	/**
	 * Gets friends of the user
	 * @param userSid String
	 * @return List<User> users returned
	 * @throws DataAccessException
	 */
	List<User> getFriends(String userSid) throws DataAccessException;
	
	/**
	 * Gets friends of the user except the users of the list
	 * @param userSid String
	 * @param users List<String>
	 * @return List<User> users returned
	 * @throws DataAccessException
	 */
	List<User> getFriendsExcept(String userSid, List<String> users) throws DataAccessException;
	
	/**
	 * Gets friends of the user except the users of the list
	 * @param words String
	 * @param userSid String
	 * @param users List<String>
	 * @return List<User> users returned
	 * @throws DataAccessException
	 */
	List<User> getFriendsExcept(String words, String userSid, List<String> users) throws DataAccessException;
	
	/**
	 * Edits email
	 * @param userSid String
	 * @param email String
	 * @throws DataAccessException
	 */
	void editEmail(String userSid, String email) throws DataAccessException;
	
	/**
	 * Edits password
	 * @param userSid String
	 * @param password String
	 * @throws DataAccessException
	 */
	void editPassword(String userSid, String password) throws DataAccessException;
}
