package services.impl;


import java.sql.Timestamp;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import services.UserDAO;
import utils.ValidationUtils;
import dto.User;
 
@Repository
public class UserDAOImpl implements UserDAO{
	
	private @Autowired GeneralDAO generalDAO;
	
	@Transactional(readOnly = false)
	public void create(String username, String email, String password) throws DataAccessException{
		try{
			String query = "INSERT INTO User (username, email, password, record_date) VALUES (?, ?, ?, ?)";
			Timestamp recordDate = new Timestamp(System.currentTimeMillis());
			Object[] params = new Object[]{
				username, email, password, recordDate
			};
			generalDAO.getJdbcTemplate().update(query, params);
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public User get(String userSid) throws DataAccessException{
		try{
			String query = "SELECT * FROM user WHERE user.username=?";
			Object[] params = new Object[]{
				userSid
			};
			return generalDAO.getJdbcTemplate().queryForObject(query, params, new BeanPropertyRowMapper<User>(User.class));
		}
		catch (EmptyResultDataAccessException e) {
			return null;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public boolean exists(String username, String password) throws DataAccessException{
		try{
			String query = "SELECT COUNT(*) FROM user WHERE username = _utf8 ? COLLATE utf8_bin AND password = ? AND ended_date IS NULL";
			Object[] params = new Object[]{
				username, password
			};
			return generalDAO.getJdbcTemplate().queryForObject(query, params, Integer.class) == 1;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public boolean usernameAndEmailAvailable(String username, String email) throws DataAccessException{
		try{
			String query = "SELECT COUNT(*) FROM user WHERE username = ? OR email = ?";
			Object[] params = new Object[]{
				username, email
			};
			return generalDAO.getJdbcTemplate().queryForObject(query, params, Integer.class) == 0;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public boolean emailAvailable(String email) throws DataAccessException{
		try{
			String query = "SELECT COUNT(*) FROM user WHERE email = ?";
			Object[] params = new Object[]{
				email
			};
			return generalDAO.getJdbcTemplate().queryForObject(query, params, Integer.class) == 0;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public List<User> getUserNotFriendsOrderAlphabetic(String userSid) throws DataAccessException {
		try{
			String query = "SELECT * FROM user WHERE username!=? AND NOT EXISTS (SELECT * FROM friend WHERE (friend.user_sid1=username AND friend.user_sid2=?) OR (friend.user_sid1=? AND friend.user_sid2=username)) ORDER BY username ASC";
			Object[] params = new Object[]{
				userSid, userSid, userSid
			};
			List<User> userList = generalDAO.getJdbcTemplate().query(query, params, new BeanPropertyRowMapper<User>(User.class));
			return userList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public List<User> getUserNotFriendsOrderAlphabetic(String userSid, String words) throws DataAccessException {
		try{
			String query = "SELECT * FROM user WHERE username!=? AND NOT EXISTS (SELECT * FROM friend WHERE (friend.user_sid1=username AND friend.user_sid2=?) OR (friend.user_sid1=? AND friend.user_sid2=username)) AND username LIKE ? ORDER BY username ASC";
			Object[] params = new Object[]{
				userSid, userSid, userSid, "%"+words+"%"
			};
			List<User> userList = generalDAO.getJdbcTemplate().query(query, params, new BeanPropertyRowMapper<User>(User.class));
			return userList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public List<User> getFriends(String userSid) throws DataAccessException {
		try{
			String query = "SELECT * FROM user WHERE EXISTS (SELECT * FROM friend WHERE ((friend.user_sid1=username AND friend.user_sid2=?) OR (friend.user_sid1=? AND friend.user_sid2=username))" +
					" AND (friend.blocker IS NULL OR friend.blocker=?)) ORDER BY username ASC";
			Object[] params = new Object[]{
				userSid, userSid, userSid
			};
			List<User> userList = generalDAO.getJdbcTemplate().query(query, params, new BeanPropertyRowMapper<User>(User.class));
			return userList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public List<User> getFriendsExcept(String userSid, List<String> users) throws DataAccessException {
		try{
			String usersParams = "";
			if (!ValidationUtils.isEmptyOrNull(users)){
				usersParams = " username NOT IN (";
				int size = users.size();
				int cont = 1;
				for (String user : users){
					usersParams = usersParams + "'" + user + "'";
					if (cont != size){
						 usersParams = usersParams + ", ";
					}
					else{
						 usersParams = usersParams + ") AND";
					}
					cont++;
				}
			}
			String query = "SELECT * FROM user WHERE"+usersParams+" EXISTS (SELECT * FROM friend WHERE ((friend.user_sid1=username AND friend.user_sid2=?) OR (friend.user_sid1=? AND friend.user_sid2=username))" +
					" AND (friend.blocker IS NULL OR friend.blocker=?)) ORDER BY username ASC";
			Object[] params = new Object[]{
				userSid, userSid, userSid
			};
			List<User> userList = generalDAO.getJdbcTemplate().query(query, params, new BeanPropertyRowMapper<User>(User.class));
			return userList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public List<User> getFriendsExcept(String words, String userSid, List<String> users) throws DataAccessException {
		try{
			String usersParams = "";
			if (!ValidationUtils.isEmptyOrNull(users)){
				usersParams = " username NOT IN (";
				int size = users.size();
				int cont = 1;
				for (String user : users){
					usersParams = usersParams + "'" + user + "'";
					if (cont != size){
						 usersParams = usersParams + ", ";
					}
					else{
						 usersParams = usersParams + ") AND";
					}
					cont++;
				}
			}
			String query = "SELECT * FROM user WHERE"+usersParams+" EXISTS (SELECT * FROM friend WHERE ((friend.user_sid1=username AND friend.user_sid2=?) OR (friend.user_sid1=? AND friend.user_sid2=username))" +
					" AND (friend.blocker IS NULL OR friend.blocker=?)) AND username LIKE ? ORDER BY username ASC";
			Object[] params = new Object[]{
				userSid, userSid, userSid, "%"+words+"%"
			};
			List<User> userList = generalDAO.getJdbcTemplate().query(query, params, new BeanPropertyRowMapper<User>(User.class));
			return userList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = false)
	public void editEmail(String userSid, String email) throws DataAccessException {
		try{
			String query = "UPDATE user SET email=?, record_date=record_date WHERE username=?";
			Object[] params = new Object[]{
				email, userSid
			};
			generalDAO.getJdbcTemplate().update(query, params);
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = false)
	public void editPassword(String userSid, String password) throws DataAccessException {
		try{
			String query = "UPDATE user SET password=?, record_date=record_date WHERE username=?";
			Object[] params = new Object[]{
				password, userSid
			};
			generalDAO.getJdbcTemplate().update(query, params);
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
}